@@ -1,15 +1,23 @@
-use std::ops::DerefMut;
+use std::collections::HashMap;
+use std::ops::{Deref, DerefMut};
 use std::sync::{Arc, LockResult, MutexGuard};
+use image::{DynamicImage, Rgba};
+use rusttype::Scale;
 use serde_json::{Map, Value};
 use crate::core::{ButtonPanel, UniqueButton};
-use crate::{ModuleManager, SDCore};
+use crate::{Config, ModuleManager, SDCore, SocketManager};
 use crate::util::{add_array_function, button_to_raw, change_from_path, convert_value_to_path, deserialize_panel, make_button_unique, panel_to_raw, remove_array_function, serialize_panel, set_value_function};
 use serde::de::Error as DeError;
 use serde_json::Error as JSONError;
-use crate::modules::events::SDEvent;
-use crate::modules::{features_to_vec, UniqueSDModule};
+use crate::core::button::{Button, parse_unique_button_to_component};
+use crate::font::get_font_from_collection;
+use crate::modules::events::{core_event_to_global, SDCoreEvent};
+use crate::modules::{features_to_vec, send_core_event_to_modules, UniqueSDModule};
 use crate::modules::components::{UIPathValue, UIValue};
-use crate::threads::streamdeck::StreamDeckCommand;
+use crate::socket::send_event_to_socket;
+use crate::thread::DeviceThreadCommunication;
+use crate::thread::rendering::{draw_background, draw_foreground, draw_missing_texture, RendererComponent};
+use crate::thread::util::{image_from_solid, render_aligned_text_on_image, TextAlignment};
 use crate::versions::SUPPORTED_FEATURES;
 
 /// Handle that's given out to a module to perform actions on the core
@@ -73,12 +81,24 @@ impl CoreHandle {
         self.core.clone()
     }
 
+    /// Returns config reference
+    pub fn config(&self) -> Arc<Config> {
+        self.required_feature("config");
+        self.core.config.clone()
+    }
+
     /// Returns module manager reference
     pub fn module_manager(&self) -> Arc<ModuleManager> {
         self.required_feature("module_manager");
         self.core.module_manager.clone()
     }
 
+    /// Returns socket manager reference
+    pub fn socket_manager(&self) -> Arc<SocketManager> {
+        self.required_feature("socket_api");
+        self.core.socket_manager.clone()
+    }
+
     /// Returns current stack lock
     pub fn current_stack(&self) -> LockResult<MutexGuard<'_, Vec<ButtonPanel>>> {
         self.required_feature("core");
@@ -109,35 +129,23 @@ pub fn set_button(core: &CoreHandle, key: u8, button: UniqueButton) -> bool {
 
         drop(handle);
 
-        core.core.mark_for_redraw();
-
         if let Some(previous_button) = previous_button {
-            for module in core.module_manager().get_module_list() {
-                if module.name() == core.module_name {
-                    continue;
-                }
-
-                module.event(core.clone_for(&module), SDEvent::ButtonUpdated {
-                    key,
-                    panel: screen.clone(),
-                    new_button: button.clone(),
-                    old_button: previous_button.clone()
-                });
-            }
+            send_core_event_to_modules(core, SDCoreEvent::ButtonUpdated {
+                key,
+                panel: screen.clone(),
+                new_button: button.clone(),
+                old_button: previous_button.clone()
+            }, core.module_manager().get_module_list().into_iter());
         } else {
-            for module in core.module_manager().get_module_list() {
-                if module.name() == core.module_name {
-                    continue;
-                }
-
-                module.event(core.clone_for(&module), SDEvent::ButtonAdded {
-                    key,
-                    panel: screen.clone(),
-                    added_button: button.clone()
-                });
-            }
+            send_core_event_to_modules(core, SDCoreEvent::ButtonAdded {
+                key,
+                panel: screen.clone(),
+                added_button: button.clone()
+            }, core.module_manager().get_module_list().into_iter());
         }
 
+        core.core.mark_for_redraw();
+
         true
     } else {
         false
@@ -152,20 +160,14 @@ pub fn clear_button(core: &CoreHandle, key: u8) -> bool {
         if let Some(button) = handle.buttons.remove(&key) {
             drop(handle);
 
+            send_core_event_to_modules(core, SDCoreEvent::ButtonDeleted {
+                key,
+                panel: screen.clone(),
+                deleted_button: button.clone()
+            }, core.module_manager().get_module_list().into_iter());
+
             core.core.mark_for_redraw();
 
-            for module in core.module_manager().get_module_list() {
-                if module.name() == core.module_name {
-                    continue;
-                }
-
-                module.event(core.clone_for(&module), SDEvent::ButtonDeleted {
-                    key,
-                    panel: screen.clone(),
-                    deleted_button: button.clone()
-                });
-            }
-
             true
         } else {
             false
@@ -178,6 +180,9 @@ pub fn clear_button(core: &CoreHandle, key: u8) -> bool {
 /// Adds component onto a button, returns success boolean
 pub fn add_component(core: &CoreHandle, key: u8, component_name: &str) -> bool {
     core.required_feature("core_methods");
+
+    let module_manager = core.module_manager();
+
     if let Some(screen) = get_current_screen(&core) {
         let handle = screen.read().unwrap();
         if let Some(button) = handle.buttons.get(&key).cloned() {
@@ -187,35 +192,24 @@ pub fn add_component(core: &CoreHandle, key: u8, component_name: &str) -> bool {
             drop(handle);
 
             if !button_handle.component_names().contains(&component_name.to_string()) {
-                let components = core.module_manager().get_components_list_by_modules();
+                let components = module_manager.read_component_map();
 
-                for (module, component_list) in components {
-                    for (component, _) in component_list {
-                        if component == component_name {
-                            let module = core.module_manager().get_module(&module).unwrap();
+                if let Some((_, module)) = components.get(component_name) {
+                    module.add_component(core.clone_for(&module), button_handle.deref_mut(), component_name);
 
-                            module.add_component(core.clone_for(&module), button_handle.deref_mut(), &component);
+                    drop(button_handle);
+                    drop(components);
 
-                            core.core.mark_for_redraw();
+                    send_core_event_to_modules(core, SDCoreEvent::ButtonUpdated {
+                        key,
+                        panel: screen.clone(),
+                        new_button: button.clone(),
+                        old_button: previous.clone()
+                    }, core.module_manager().get_module_list().into_iter());
 
-                            drop(button_handle);
+                    core.core.mark_for_redraw();
 
-                            for module in core.module_manager().get_module_list() {
-                                if module.name() == core.module_name {
-                                    continue;
-                                }
-
-                                module.event(core.clone_for(&module), SDEvent::ButtonUpdated {
-                                    key,
-                                    panel: screen.clone(),
-                                    new_button: button.clone(),
-                                    old_button: previous.clone()
-                                });
-                            }
-
-                            return true;
-                        }
-                    }
+                    return true;
                 }
             }
         }
@@ -227,6 +221,9 @@ pub fn add_component(core: &CoreHandle, key: u8, component_name: &str) -> bool {
 /// Gets component values from a component on a button
 pub fn get_component_values(core: &CoreHandle, key: u8, component_name: &str) -> Option<Vec<UIValue>> {
     core.required_feature("core_methods");
+
+    let module_manager = core.module_manager();
+
     if let Some(screen) = get_current_screen(&core) {
         let handle = screen.read().unwrap();
         if let Some(button) = handle.buttons.get(&key).cloned() {
@@ -234,16 +231,10 @@ pub fn get_component_values(core: &CoreHandle, key: u8, component_name: &str) ->
             drop(handle);
 
             if button_handle.component_names().contains(&component_name.to_string()) {
-                let components = core.module_manager().get_components_list_by_modules();
+                let components = module_manager.read_component_map();
 
-                for (module, component_list) in components {
-                    for (component, _) in component_list {
-                        if component == component_name {
-                            let module = core.module_manager().get_module(&module).unwrap();
-
-                            return Some(module.component_values(core.clone_for(&module), button_handle.deref_mut(), &component));
-                        }
-                    }
+                if let Some((_, module)) = components.get(component_name) {
+                    return Some(module.component_values(core.clone_for(&module), button_handle.deref_mut(), component_name));
                 }
             }
         }
@@ -264,6 +255,9 @@ pub fn get_component_values_with_paths(core: &CoreHandle, key: u8, component_nam
 /// Sets component values based on changes for component on a button
 pub fn set_component_value(core: &CoreHandle, key: u8, component_name: &str, value: Vec<UIValue>) -> bool {
     core.required_feature("core_methods");
+
+    let module_manager = core.module_manager();
+
     if let Some(screen) = get_current_screen(&core) {
         let handle = screen.read().unwrap();
         if let Some(button) = handle.buttons.get(&key).cloned() {
@@ -273,33 +267,23 @@ pub fn set_component_value(core: &CoreHandle, key: u8, component_name: &str, val
             drop(handle);
 
             if button_handle.component_names().contains(&component_name.to_string()) {
-                let components = core.module_manager().get_components_list_by_modules();
+                let components = module_manager.read_component_map();
 
-                for (module, component_list) in components {
-                    for (component, _) in component_list {
-                        if component == component_name {
-                            let module = core.module_manager().get_module(&module).unwrap();
-                            module.set_component_value(core.clone_for(&module), button_handle.deref_mut(), &component, value);
-                            drop(button_handle);
+                if let Some((_, module)) = components.get(component_name) {
+                    module.set_component_value(core.clone_for(&module), button_handle.deref_mut(), component_name, value);
+                    drop(button_handle);
+                    drop(components);
 
-                            core.core.mark_for_redraw();
+                    send_core_event_to_modules(core, SDCoreEvent::ButtonUpdated {
+                        key,
+                        panel: screen.clone(),
+                        new_button: button.clone(),
+                        old_button: previous.clone()
+                    }, core.module_manager().get_module_list().into_iter());
 
-                            for module in core.module_manager().get_module_list() {
-                                if module.name() == core.module_name {
-                                    continue;
-                                }
+                    core.core.mark_for_redraw();
 
-                                module.event(core.clone_for(&module), SDEvent::ButtonUpdated {
-                                    key,
-                                    panel: screen.clone(),
-                                    new_button: button.clone(),
-                                    old_button: previous.clone()
-                                });
-                            }
-
-                            return true;
-                        }
-                    }
+                    return true;
                 }
             }
         }
@@ -368,6 +352,9 @@ pub fn set_component_value_by_path(core: &CoreHandle, key: u8, component_name: &
 /// Removes component from a button
 pub fn remove_component(core: &CoreHandle, key: u8, component_name: &str) -> bool {
     core.required_feature("core_methods");
+
+    let module_manager = core.module_manager();
+
     if let Some(screen) = get_current_screen(&core) {
         let handle = screen.read().unwrap();
         if let Some(button) = handle.buttons.get(&key).cloned() {
@@ -377,35 +364,24 @@ pub fn remove_component(core: &CoreHandle, key: u8, component_name: &str) -> boo
             drop(handle);
 
             if button_handle.component_names().contains(&component_name.to_string()) {
-                let components = core.module_manager().get_components_list_by_modules();
+                let components = module_manager.read_component_map();
 
-                for (module, component_list) in components {
-                    for (component, _) in component_list {
-                        if component == component_name {
-                            let module = core.module_manager().get_module(&module).unwrap();
+                if let Some((_, module)) = components.get(component_name) {
+                    module.remove_component(core.clone_for(&module), button_handle.deref_mut(), component_name);
 
-                            module.remove_component(core.clone_for(&module), button_handle.deref_mut(), &component);
+                    drop(button_handle);
+                    drop(components);
 
-                            drop(button_handle);
+                    send_core_event_to_modules(core, SDCoreEvent::ButtonUpdated {
+                        key,
+                        panel: screen.clone(),
+                        new_button: button.clone(),
+                        old_button: previous.clone()
+                    }, core.module_manager().get_module_list().into_iter());
 
-                            core.core.mark_for_redraw();
+                    core.core.mark_for_redraw();
 
-                            for module in core.module_manager().get_module_list() {
-                                if module.name() == core.module_name {
-                                    continue;
-                                }
-
-                                module.event(core.clone_for(&module), SDEvent::ButtonUpdated {
-                                    key,
-                                    panel: screen.clone(),
-                                    new_button: button.clone(),
-                                    old_button: previous.clone()
-                                });
-                            }
-
-                            return true;
-                        }
-                    }
+                    return true;
                 }
             }
         }
@@ -414,6 +390,19 @@ pub fn remove_component(core: &CoreHandle, key: u8, component_name: &str) -> boo
     false
 }
 
+pub fn paste_button(core: &CoreHandle, key: u8, reference_button: &Button) -> bool {
+    let mut new_button = Button::new();
+
+    let responsible_modules = core.module_manager().get_modules_for_declared_components(reference_button.component_names().as_slice());
+    for module in responsible_modules {
+        module.paste_component(core.clone_for(&module), reference_button, &mut new_button);
+    }
+
+    println!("resulting button: {:?}", new_button);
+
+    set_button(core, key, make_button_unique(new_button))
+}
+
 /// Pushes new panel into the stack
 pub fn push_screen(core: &CoreHandle, screen: ButtonPanel) {
     core.required_feature("core_methods");
@@ -422,17 +411,11 @@ pub fn push_screen(core: &CoreHandle, screen: ButtonPanel) {
     stack.push(screen.clone());
     drop(stack);
 
-    for module in core.module_manager().get_module_list() {
-        if module.name() == core.module_name {
-            continue;
-        }
+    send_core_event_to_modules(core, SDCoreEvent::PanelPushed {
+        new_panel: screen.clone()
+    }, core.module_manager().get_module_list().into_iter());
 
-        module.event(core.clone_for(&module), SDEvent::PanelPushed {
-            new_panel: screen.clone()
-        });
-    }
-
-    core.core().mark_for_redraw();
+    core.core.mark_for_redraw();
 }
 
 /// Pops panel from stack
@@ -444,18 +427,12 @@ pub fn pop_screen(core: &CoreHandle) {
     drop(stack);
 
     if let Some(old_panel) = old_panel {
-        for module in core.module_manager().get_module_list() {
-            if module.name() == core.module_name {
-                continue;
-            }
-
-            module.event(core.clone_for(&module), SDEvent::PanelPopped {
-                popped_panel: old_panel.clone()
-            })
-        }
+        send_core_event_to_modules(core, SDCoreEvent::PanelPopped {
+            popped_panel: old_panel.clone()
+        }, core.module_manager().get_module_list().into_iter());
     }
 
-    core.core().mark_for_redraw();
+    core.core.mark_for_redraw();
 }
 
 /// Returns first panel of the stack for saving purposes
@@ -487,17 +464,11 @@ pub fn reset_stack(core: &CoreHandle, panel: ButtonPanel) {
     stack.push(panel.clone());
     drop(stack);
 
-    for module in core.module_manager().get_module_list() {
-        if module.name() == core.module_name {
-            continue;
-        }
+    send_core_event_to_modules(core, SDCoreEvent::StackReset {
+        new_panel: panel.clone()
+    }, core.module_manager().get_module_list().into_iter());
 
-        module.event(core.clone_for(&module), SDEvent::StackReset {
-            new_panel: panel.clone()
-        });
-    }
-
-    core.core().mark_for_redraw();
+    core.core.mark_for_redraw();
 }
 
 /// Clears the stack, attempts to deserialize provided panel value into an actual panel and then pushes it into the stack
@@ -511,17 +482,11 @@ pub fn load_panels_from_value(core: &CoreHandle, panels: Value) -> Result<(), JS
             stack.push(panel.clone());
             drop(stack);
 
-            for module in core.module_manager().get_module_list() {
-                if module.name() == core.module_name {
-                    continue;
-                }
+            send_core_event_to_modules(core, SDCoreEvent::StackReset {
+                new_panel: panel.clone()
+            }, core.module_manager().get_module_list().into_iter());
 
-                module.event(core.clone_for(&module), SDEvent::StackReset {
-                    new_panel: panel.clone()
-                });
-            }
-
-            core.core().mark_for_redraw();
+            core.core.mark_for_redraw();
 
             Ok(())
         }
@@ -534,29 +499,17 @@ pub fn load_panels_from_value(core: &CoreHandle, panels: Value) -> Result<(), JS
 /// Triggers button down event on all modules
 pub fn button_down(core: &CoreHandle, key: u8) {
     core.required_feature("core_methods");
-    for module in core.module_manager().get_module_list() {
-        if module.name() == core.module_name {
-            continue;
-        }
-
-        module.event(core.clone_for(&module), SDEvent::ButtonDown {
-            key
-        })
-    }
+    send_core_event_to_modules(core, SDCoreEvent::ButtonDown {
+        key
+    }, core.module_manager().get_module_list().into_iter());
 }
 
 /// Triggers button up event on all modules
 pub fn button_up(core: &CoreHandle, key: u8) {
     core.required_feature("core_methods");
-    for module in core.module_manager().get_module_list() {
-        if module.name() == core.module_name {
-            continue;
-        }
-
-        module.event(core.clone_for(&module), SDEvent::ButtonUp {
-            key
-        })
-    }
+    send_core_event_to_modules(core, SDCoreEvent::ButtonUp {
+        key
+    }, core.module_manager().get_module_list().into_iter());
 
     button_action(core, key);
 }
@@ -568,17 +521,17 @@ pub fn button_action(core: &CoreHandle, key: u8) {
         let handle = screen.read().unwrap();
         if let Some(button) = handle.buttons.get(&key).cloned() {
             drop(handle);
-            for module in core.module_manager().get_modules_for_components(button.read().unwrap().component_names().as_slice()) {
-                if module.name() == core.module_name {
-                    continue;
-                }
 
-                module.event(core.clone_for(&module), SDEvent::ButtonAction {
-                    key,
-                    panel: screen.clone(),
-                    pressed_button: button.clone()
-                })
-            }
+            let event = SDCoreEvent::ButtonAction {
+                key,
+                panel: screen.clone(),
+                pressed_button: button.clone()
+            };
+
+            send_core_event_to_modules(&core, event.clone(), core.module_manager().get_modules_for_components(button.read().unwrap().component_names().as_slice()).into_iter());
+            send_event_to_socket(&core.core.socket_manager, core_event_to_global(event, &core.core.serial_number));
+
+            core.core.mark_for_redraw();
         }
     }
 }
@@ -603,19 +556,137 @@ pub fn get_current_screen(core: &CoreHandle) -> Option<ButtonPanel> {
     }
 }
 
+pub fn get_button_images(core: &CoreHandle) -> Option<HashMap<u8, DynamicImage>> {
+    let missing = draw_missing_texture(core.core.image_size);
+    let custom = {
+        let size = core.core.image_size;
+        let font = get_font_from_collection("default").unwrap();
+        let mut frame = image_from_solid(size, Rgba([55, 55, 55, 255]));
+
+        render_aligned_text_on_image(size, &mut frame, font.deref(), "Custom", Scale::uniform(16.0), TextAlignment::Center, 0, (0.0, -8.0), (255, 255, 255, 255));
+        render_aligned_text_on_image(size, &mut frame, font.deref(), "Renderer", Scale::uniform(16.0), TextAlignment::Center, 0, (0.0, 8.0), (255, 255, 255, 255));
+
+        frame
+    };
+
+    let panel = get_current_screen(core)?;
+    let current_screen = panel.read().unwrap();
+    let buttons = current_screen.buttons.clone();
+
+    let renderers = core.core.render_manager.read_renderers();
+
+    Some(buttons.into_iter()
+        .filter_map(|(key, button)| {
+            if let Ok(component) = parse_unique_button_to_component::<RendererComponent>(&button) {
+                let modules = core.module_manager().get_modules_for_rendering(&button.read().unwrap().component_names());
+                let modules = modules.into_values()
+                    .filter(|x| !component.plugin_blacklist.contains(&x.name()))
+                    .collect::<Vec<UniqueSDModule>>();
+
+                let image = if component.renderer.is_empty() {
+                    draw_foreground(
+                        &component,
+                        &button,
+                        &modules,
+                        draw_background(
+                            &component,
+                            core,
+                            &missing
+                        ),
+                        core
+                    )
+                } else {
+                    if let Some(renderer) = renderers.get(&component.renderer) {
+                        if let Some(image) = renderer.representation(key, &button, core) {
+                            image
+                        } else {
+                            custom.clone()
+                        }
+                    } else {
+                        custom.clone()
+                    }
+                };
+
+                Some((key, image))
+            } else {
+                None
+            }
+        })
+        .collect())
+}
+
+pub fn get_button_image(core: &CoreHandle, key: u8) -> Option<DynamicImage> {
+    let missing = draw_missing_texture(core.core.image_size);
+    let custom = {
+        let size = core.core.image_size;
+        let font = get_font_from_collection("default").unwrap();
+        let mut frame = image_from_solid(size, Rgba([55, 55, 55, 255]));
+
+        render_aligned_text_on_image(size, &mut frame, font.deref(), "Custom", Scale::uniform(16.0), TextAlignment::Center, 0, (0.0, -8.0), (255, 255, 255, 255));
+        render_aligned_text_on_image(size, &mut frame, font.deref(), "Renderer", Scale::uniform(16.0), TextAlignment::Center, 0, (0.0, 8.0), (255, 255, 255, 255));
+
+        frame
+    };
+
+    let button = get_button(core, key)?;
+    let renderers = core.core.render_manager.read_renderers();
+
+    if let Ok(component) = parse_unique_button_to_component::<RendererComponent>(&button) {
+        let modules = core.module_manager().get_modules_for_rendering(&button.read().unwrap().component_names());
+        let modules = modules.into_values()
+            .filter(|x| !component.plugin_blacklist.contains(&x.name()))
+            .collect::<Vec<UniqueSDModule>>();
+
+        let image = if component.renderer.is_empty() {
+            draw_foreground(
+                &component,
+                &button,
+                &modules,
+                draw_background(
+                    &component,
+                    core,
+                    &missing
+                ),
+                core
+            )
+        } else {
+            if let Some(renderer) = renderers.get(&component.renderer) {
+                if let Some(image) = renderer.representation(key, &button, core) {
+                    image
+                } else {
+                    custom.clone()
+                }
+            } else {
+                custom.clone()
+            }
+        };
+
+        Some(image)
+    } else {
+        None
+    }
+}
+
 /// Replaces current screen with specified one
 pub fn replace_screen(core: &CoreHandle, screen: ButtonPanel) {
     core.required_feature("core_methods");
     let mut stack = core.current_stack().unwrap();
-    stack.pop();
-    stack.push(screen);
-    core.core().mark_for_redraw();
+
+    let old_panel = stack.pop();
+    stack.push(screen.clone());
+
+    send_core_event_to_modules(core, SDCoreEvent::PanelReplaced {
+        old_panel,
+        new_panel: screen
+    }, core.module_manager().get_module_list().into_iter());
+
+    core.core.mark_for_redraw();
 }
 
 /// Sets brightness of the streamdeck to specified (Range from 0 to 100)
 pub fn set_brightness(core: &CoreHandle, brightness: u8) {
     core.required_feature("core_methods");
-    core.core().send_commands(vec![StreamDeckCommand::SetBrightness(brightness)]);
+    core.core().send_commands(vec![DeviceThreadCommunication::SetBrightness(brightness)]);
 
     let core = core.core();
     let mut handle = core.device_config.write().unwrap();
