@@ -1,8 +1,8 @@
 use std::collections::HashMap;
 use std::path::PathBuf;
 use std::str::FromStr;
-use serde::{Serialize, Deserialize};
-use crate::threads::rendering::{Color, RendererComponent};
+use serde::{Deserialize, Serialize};
+use crate::thread::rendering::{Color, RendererComponent};
 
 /// Component definition
 #[derive(Serialize, Deserialize, Clone, Debug)]
@@ -22,6 +22,7 @@ pub struct ComponentDefinition {
 pub struct UIField {
     pub name: String,
     pub display_name: String,
+    pub description: String,
     pub ty: UIFieldType,
     pub default_value: UIFieldValue<UIValue>
 }
@@ -31,6 +32,7 @@ pub struct UIField {
 pub struct UIValue {
     pub name: String,
     pub display_name: String,
+    pub description: String,
     pub ty: UIFieldType,
     pub value: UIFieldValue<UIValue>,
 }
@@ -41,6 +43,7 @@ pub struct UIPathValue {
     pub name: String,
     pub path: String,
     pub display_name: String,
+    pub description: String,
     pub ty: UIFieldType,
     pub value: UIFieldValue<UIPathValue>,
 }
@@ -88,6 +91,9 @@ pub enum UIFieldType {
 
     /// Checkbox
     Checkbox {
+        /// If checkbox should appear disabled in UI
+        ///
+        /// Note: Doesn't actually prevent setting the checkbox, so do internal checks if value should be changed
         disabled: bool
     },
 
@@ -482,6 +488,7 @@ impl From<UIFieldValue<UIPathValue>> for UIFieldValue<UIValue> {
                     .map(|x| UIValue {
                         name: x.name,
                         display_name: x.display_name,
+                        description: x.description,
                         ty: x.ty,
                         value: x.value.into()
                     })
@@ -494,6 +501,7 @@ impl From<UIFieldValue<UIPathValue>> for UIFieldValue<UIValue> {
                         .map(|x| UIValue {
                             name: x.name,
                             display_name: x.display_name,
+                            description: x.description,
                             ty: x.ty,
                             value: x.value.into()
                         })
@@ -516,6 +524,7 @@ impl From<UIPathValue> for UIValue {
         UIValue {
             name: value.name,
             display_name: value.display_name,
+            description: value.description,
             ty: value.ty,
             value: value.value.into()
         }
