@@ -1,25 +1,32 @@
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::sync::RwLock;
 use rand::distributions::Alphanumeric;
 use rand::Rng;
-use serde::{Serialize, Deserialize};
+use serde::{Deserialize, Serialize};
 use crate::core::button::{Button, Component, parse_button_to_component, parse_unique_button_to_component};
 use crate::core::{ButtonPanel, RawButtonPanel};
 use crate::core::methods::{CoreHandle, get_stack, pop_screen, push_screen};
 use crate::modules::components::{ComponentDefinition, map_ui_values, UIFieldType, UIFieldValue, UIValue};
-use crate::modules::events::SDEvent;
+use crate::modules::events::SDCoreEvent;
 use crate::modules::{PluginMetadata, SDModule};
-use crate::threads::rendering::{ButtonBackground, ButtonText, RendererComponent};
-use crate::util::{button_to_raw, make_panel_unique};
-use crate::util::rendering::TextAlignment;
-use crate::versions::{CORE, CORE_METHODS, EVENTS, MODULE_MANAGER};
+use crate::thread::rendering::{ButtonBackground, ButtonText, RendererComponentBuilder};
+use crate::util::{button_to_raw, make_panel_unique, straight_copy};
+use crate::thread::util::TextAlignment;
+use crate::versions::{CORE, CORE_METHODS, CORE_EVENTS, MODULE_MANAGER};
 
 const MODULE_NAME: &str = "core/folder";
 
-#[derive(Default, Debug)]
+#[derive(Debug)]
 pub struct FolderModule {
-    folder_stack: RwLock<Vec<String>>,
-    folder_references: RwLock<HashMap<String, ButtonPanel>>
+    folder_references: RwLock<HashMap<String, ButtonPanel>>,
+}
+
+impl Default for FolderModule {
+    fn default() -> Self {
+        Self {
+            folder_references: Default::default(),
+        }
+    }
 }
 
 impl SDModule for FolderModule {
@@ -33,64 +40,55 @@ impl SDModule for FolderModule {
         map.insert(FolderComponent::NAME.to_string(), ComponentDefinition {
             display_name: "Folder".to_string(),
             description: "Opens folder".to_string(),
-            default_looks: RendererComponent {
-                background: ButtonBackground::Solid((0, 50, 200, 255)),
-                text: vec![
-                    ButtonText {
-                        text: "Folder".to_string(),
-                        font: "default".to_string(),
-                        scale: (22.0, 22.0),
-                        alignment: TextAlignment::Center,
-                        padding: 0,
-                        offset: (0.0, 0.0),
-                        color: (255, 255, 255, 255),
-                        shadow: None
-                    }
-                ],
-                to_cache: true
-            }
+            default_looks: RendererComponentBuilder::new()
+                .background(ButtonBackground::Solid((0, 50, 200, 255)))
+                .add_text(ButtonText {
+                    text: "Folder".to_string(),
+                    font: "default".to_string(),
+                    scale: (22.0, 22.0),
+                    alignment: TextAlignment::Center,
+                    padding: 0,
+                    offset: (0.0, 0.0),
+                    color: (255, 255, 255, 255),
+                    shadow: None
+                })
+                .build()
         });
 
         map.insert(FolderLinkComponent::NAME.to_string(), ComponentDefinition {
             display_name: "Folder Link".to_string(),
             description: "Opens existing folders in symlink style".to_string(),
-            default_looks: RendererComponent {
-                background: ButtonBackground::Solid((0, 50, 200, 255)),
-                text: vec![
-                    ButtonText {
-                        text: "⇲".to_string(),
-                        font: "default".to_string(),
-                        scale: (32.0, 32.0),
-                        alignment: TextAlignment::BottomRight,
-                        padding: 7,
-                        offset: (0.0, 0.0),
-                        color: (255, 255, 255, 255),
-                        shadow: None
-                    }
-                ],
-                to_cache: true
-            }
+            default_looks: RendererComponentBuilder::new()
+                .background(ButtonBackground::Solid((0, 50, 200, 255)))
+                .add_text(ButtonText {
+                                text: "⇲".to_string(),
+                                font: "default".to_string(),
+                                scale: (32.0, 32.0),
+                                alignment: TextAlignment::BottomRight,
+                                padding: 7,
+                                offset: (0.0, 0.0),
+                                color: (255, 255, 255, 255),
+                                shadow: None
+                })
+                .build()
         });
 
         map.insert(FolderUpComponent::NAME.to_string(), ComponentDefinition {
             display_name: "Folder Up".to_string(),
             description: "Back button for folders".to_string(),
-            default_looks: RendererComponent {
-                background: ButtonBackground::Solid((50, 50, 50, 255)),
-                text: vec![
-                    ButtonText {
-                        text: "Back".to_string(),
-                        font: "default".to_string(),
-                        scale: (22.0, 22.0),
-                        alignment: TextAlignment::Center,
-                        padding: 0,
-                        offset: (0.0, 0.0),
-                        color: (255, 255, 255, 255),
-                        shadow: None
-                    }
-                ],
-                to_cache: true
-            }
+            default_looks: RendererComponentBuilder::new()
+                .background(ButtonBackground::Solid((50, 50, 50, 255)))
+                .add_text(ButtonText {
+                    text: "Back".to_string(),
+                    font: "default".to_string(),
+                    scale: (22.0, 22.0),
+                    alignment: TextAlignment::Center,
+                    padding: 0,
+                    offset: (0.0, 0.0),
+                    color: (255, 255, 255, 255),
+                    shadow: None
+                })
+                .build()
         });
 
         map
@@ -99,22 +97,26 @@ impl SDModule for FolderModule {
     fn add_component(&self, core: CoreHandle, button: &mut Button, name: &str) {
         match name {
             FolderComponent::NAME => {
-                let folder_id = self.new_folder(&core);
+                if !button.contains(FolderLinkComponent::NAME) {
+                    let folder_id = self.new_folder(&core);
 
-                button.insert_component(
-                    FolderComponent {
-                        id: folder_id,
-                        name: "Folder".to_string()
-                    }
-                ).ok();
+                    button.insert_component(
+                        FolderComponent {
+                            id: folder_id,
+                            name: "Folder".to_string()
+                        }
+                    ).ok();
+                }
             }
 
             FolderLinkComponent::NAME => {
-                button.insert_component(
-                    FolderLinkComponent {
-                        id: "".to_string()
-                    }
-                ).ok();
+                if !button.contains(FolderComponent::NAME) {
+                    button.insert_component(
+                        FolderLinkComponent {
+                            id: "".to_string()
+                        }
+                    ).ok();
+                }
             }
 
             FolderUpComponent::NAME => {
@@ -131,7 +133,7 @@ impl SDModule for FolderModule {
         match name {
             FolderComponent::NAME => {
                 if let Ok(component) = parse_button_to_component::<FolderComponent>(button) {
-                    self.delete_folder_recursively(&core, &component.id);
+                    self.delete_folder_recursively(&core, &component.id, &mut HashSet::new());
                 }
 
                 button.remove_component::<FolderComponent>();
@@ -149,6 +151,28 @@ impl SDModule for FolderModule {
         }
     }
 
+    fn paste_component(&self, core: CoreHandle, reference_button: &Button, new_button: &mut Button) {
+        straight_copy(reference_button, new_button, FolderLinkComponent::NAME);
+        straight_copy(reference_button, new_button, FolderUpComponent::NAME);
+
+        if let Ok(component) = parse_button_to_component::<FolderComponent>(reference_button) {
+            let new_name = self.random_unique_name(&core);
+
+            let reference_folder = self.get_folder(&core, &component.id).unwrap_or_else(|| RawButtonPanel {
+                display_name: "Folder".to_string(),
+                data: Default::default(),
+                buttons: Default::default()
+            });
+
+            self.update_folder(&core, new_name.clone(), reference_folder);
+
+            new_button.insert_component(FolderComponent {
+                id: new_name,
+                name: component.name
+            }).ok();
+        }
+    }
+
     fn component_values(&self, core: CoreHandle, button: &Button, component: &str) -> Vec<UIValue> {
         match component {
             FolderComponent::NAME => {
@@ -157,12 +181,14 @@ impl SDModule for FolderModule {
                         UIValue {
                             name: "id".to_string(),
                             display_name: "ID".to_string(),
+                            description: "ID of the folder".to_string(),
                             ty: UIFieldType::Label,
                             value: UIFieldValue::Label(component.id)
                         },
                         UIValue {
                             name: "name".to_string(),
                             display_name: "Folder Name".to_string(),
+                            description: "Name that will appear in breadcrumbs of the stack".to_string(),
                             ty: UIFieldType::InputFieldString,
                             value: UIFieldValue::InputFieldString(component.name)
                         }
@@ -187,6 +213,7 @@ impl SDModule for FolderModule {
                         UIValue {
                             name: "id".to_string(),
                             display_name: "ID".to_string(),
+                            description: "Folder to link to".to_string(),
                             ty: UIFieldType::Choice(choices),
                             value: UIFieldValue::Choice(choice)
                         }
@@ -262,64 +289,54 @@ impl SDModule for FolderModule {
         ]
     }
 
-    fn event(&self, core: CoreHandle, event: SDEvent) {
+    fn event(&self, core: CoreHandle, event: SDCoreEvent) {
         match event {
-            SDEvent::ButtonAdded { key, added_button, .. } => {
-                let mut stack = self.folder_stack.read().unwrap().clone();
+            SDCoreEvent::ButtonAdded { key, added_button, panel } |
+            SDCoreEvent::ButtonUpdated { key, new_button: added_button, panel, .. } => {
+                let panel = panel.read().unwrap();
 
-                if let Some(id) = stack.pop() {
-                    if let Some(mut contents) = self.get_folder(&core, &id) {
+                if let Ok(stack_data) = serde_json::from_value::<FolderStackData>(panel.data.clone()) {
+                    if let Some(mut contents) = self.get_folder(&core, &stack_data.folder_id) {
                         contents.buttons.insert(key, button_to_raw(&added_button));
-                        self.update_folder(&core, id, contents);
+                        self.update_folder(&core, stack_data.folder_id, contents);
                     }
                 }
             }
 
-            SDEvent::ButtonUpdated { key, new_button, .. } => {
-                let mut stack = self.folder_stack.read().unwrap().clone();
+            SDCoreEvent::ButtonDeleted { key, deleted_button, panel, .. } => {
+                let panel = panel.read().unwrap();
 
-                if let Some(id) = stack.pop() {
-                    if let Some(mut contents) = self.get_folder(&core, &id) {
-                        contents.buttons.insert(key, button_to_raw(&new_button));
-                        self.update_folder(&core, id, contents);
-                    }
-                }
-            }
-
-            SDEvent::ButtonDeleted { key, deleted_button, .. } => {
-                let mut stack = self.folder_stack.read().unwrap().clone();
-
-                if let Some(id) = stack.pop() {
-                    if let Some(mut contents) = self.get_folder(&core, &id) {
+                if let Ok(stack_data) = serde_json::from_value::<FolderStackData>(panel.data.clone()) {
+                    if let Some(mut contents) = self.get_folder(&core, &stack_data.folder_id) {
                         contents.buttons.remove(&key);
-                        self.update_folder(&core, id, contents);
+                        self.update_folder(&core, stack_data.folder_id, contents);
                     }
                 }
 
                 if let Ok(component) = parse_unique_button_to_component::<FolderComponent>(&deleted_button) {
-                    self.delete_folder_recursively(&core, &component.id);
+                    self.delete_folder_recursively(&core, &component.id, &mut HashSet::new());
                 }
             }
 
-            SDEvent::ButtonAction { pressed_button, .. } => {
+            SDCoreEvent::ButtonAction { pressed_button, .. } => {
                 if let Ok(_) = parse_unique_button_to_component::<FolderUpComponent>(&pressed_button) {
                     if get_stack(&core).len() > 1 {
                         pop_screen(&core);
-
-                        self.folder_stack.write().unwrap().pop();
                     }
                 } else if let Ok(folder) = parse_unique_button_to_component::<FolderComponent>(&pressed_button) {
                     let mut folder_ref_handle = self.folder_references.write().unwrap();
 
                     if let Some(panel) = folder_ref_handle.get(&folder.id).cloned() {
                         push_screen(&core, panel);
-                        self.folder_stack.write().unwrap().push(folder.id.clone());
                     } else {
                         if let Some(mut contents) = self.get_folder(&core, &folder.id) {
                             contents.display_name = folder.name;
+                            contents.data = serde_json::to_value(FolderStackData {
+                                folder_id: folder.id.to_string()
+                            }).unwrap();
+
                             let panel = make_panel_unique(contents);
                             push_screen(&core, panel.clone());
-                            self.folder_stack.write().unwrap().push(folder.id.clone());
                             folder_ref_handle.insert(folder.id, panel);
                         }
                     }
@@ -330,30 +347,20 @@ impl SDModule for FolderModule {
 
                     if let Some(panel) = folder_ref_handle.get(&folder_link.id).cloned() {
                         push_screen(&core, panel);
-                        self.folder_stack.write().unwrap().push(folder_link.id.clone());
                     } else {
-                        if let Some(contents) = self.get_folder(&core, &folder_link.id) {
+                        if let Some(mut contents) = self.get_folder(&core, &folder_link.id) {
+                            contents.data = serde_json::to_value(FolderStackData {
+                                folder_id: folder_link.id.to_string()
+                            }).unwrap();
+
                             let panel = make_panel_unique(contents);
                             push_screen(&core, panel.clone());
-                            self.folder_stack.write().unwrap().push(folder_link.id.clone());
                             folder_ref_handle.insert(folder_link.id, panel);
                         }
                     }
                 }
             }
 
-            SDEvent::PanelPushed { .. } => {
-                self.folder_stack.write().unwrap().push("unknown".to_string())
-            }
-
-            SDEvent::PanelPopped { .. } => {
-                self.folder_stack.write().unwrap().pop();
-            }
-
-            SDEvent::StackReset { .. } => {
-                self.folder_stack.write().unwrap().clear();
-            }
-
             _ => {}
         }
     }
@@ -368,7 +375,7 @@ impl SDModule for FolderModule {
                 CORE,
                 CORE_METHODS,
                 MODULE_MANAGER,
-                EVENTS
+                CORE_EVENTS
             ]
         )
     }
@@ -377,6 +384,23 @@ impl SDModule for FolderModule {
 type FolderMap = HashMap<String, RawButtonPanel>;
 
 impl FolderModule {
+    /// Generates a random name for folder
+    fn random_name(&self) -> String {
+        rand::thread_rng().sample_iter(&Alphanumeric).take(16).map(char::from).collect::<String>()
+    }
+
+    /// Generates a random name for folder and ensures it's not used anywhere
+    fn random_unique_name(&self, core: &CoreHandle) -> String {
+        let folder_list = self.list_folders(core);
+
+        let mut name = self.random_name();
+        while folder_list.get(&name).is_some() {
+            name = self.random_name();
+        }
+
+        name
+    }
+
     /// Creates a new folder in plugin data
     fn new_folder(&self, core: &CoreHandle) -> String {
         let core = core.core();
@@ -393,7 +417,7 @@ impl FolderModule {
         };
 
         loop {
-            let str = rand::thread_rng().sample_iter(&Alphanumeric).take(16).map(char::from).collect::<String>();
+            let str = self.random_name();
             if !folders.contains_key(&str) {
                 folders.insert(str.clone(), RawButtonPanel {
                     display_name: "Folder".to_string(),
@@ -477,11 +501,14 @@ impl FolderModule {
     }
 
     /// Deletes folder with all folders that are linked from the folder recursively
-    fn delete_folder_recursively(&self, core: &CoreHandle, folder_id: &str) {
+    fn delete_folder_recursively(&self, core: &CoreHandle, folder_id: &str, ids: &mut HashSet<String>) {
         if let Some(folder) = self.get_folder(core, folder_id) {
             for (_, button) in folder.buttons {
                 if let Ok(folder) = parse_button_to_component::<FolderComponent>(&button) {
-                    self.delete_folder_recursively(core, &folder.id);
+                    if !ids.contains(&folder.id) {
+                        ids.insert(folder.id.clone());
+                        self.delete_folder_recursively(core, &folder.id, ids);
+                    }
                 }
             }
 
@@ -491,7 +518,7 @@ impl FolderModule {
 }
 
 
-#[derive(Serialize, Deserialize, Default)]
+#[derive(Serialize, Deserialize)]
 pub struct FolderComponent {
     #[serde(default)]
     pub id: String,
@@ -502,7 +529,7 @@ impl Component for FolderComponent {
     const NAME: &'static str = "folder";
 }
 
-#[derive(Serialize, Deserialize, Default)]
+#[derive(Serialize, Deserialize)]
 pub struct FolderLinkComponent {
     #[serde(default)]
     pub id: String
@@ -512,9 +539,14 @@ impl Component for FolderLinkComponent {
     const NAME: &'static str = "folder_link";
 }
 
-#[derive(Serialize, Deserialize, Default)]
+#[derive(Serialize, Deserialize)]
 pub struct FolderUpComponent {}
 
 impl Component for FolderUpComponent {
     const NAME: &'static str = "folder_up";
+}
+
+#[derive(Serialize, Deserialize)]
+pub struct FolderStackData {
+    folder_id: String,
 }
\ No newline at end of file
