@@ -1,24 +1,30 @@
+//! Plugin API for loading dynamic library files
+
 use std::collections::HashMap;
 use std::ffi::OsStr;
 use std::fs;
+use std::hash::Hasher;
 use std::path::Path;
 use std::sync::Arc;
 use dlopen::Error;
 use crate::modules::{BoxedSDModule, ModuleManager, PluginMetadata, SDModule, SDModulePointer, UniqueSDModule};
 use dlopen::wrapper::{Container, WrapperApi};
 use dlopen_derive::WrapperApi;
+use image::DynamicImage;
 use crate::core::button::Button;
-use crate::core::methods::{CoreHandle, warn_for_feature};
+use crate::core::manager::CoreManager;
+use crate::core::methods::{check_feature_list_for_feature, CoreHandle, warn_for_feature};
+use crate::core::UniqueButton;
 use crate::modules::components::{ComponentDefinition, UIValue};
-use crate::modules::events::SDEvent;
+use crate::modules::events::{SDCoreEvent, SDGlobalEvent};
+use crate::RenderingManager;
 use crate::socket::SocketManager;
 use crate::versions::SUPPORTED_FEATURES;
 
 #[derive(WrapperApi)]
 struct PluginApi {
     get_metadata: extern fn() -> PluginMetadata,
-    get_module: extern fn() -> SDModulePointer,
-    register: extern fn(socket_manager: Arc<SocketManager>),
+    get_module: extern fn(socket_manager: Arc<SocketManager>, render_manager: Arc<RenderingManager>) -> SDModulePointer,
 }
 
 #[allow(dead_code)]
@@ -45,6 +51,10 @@ impl SDModule for PluginProxy {
         self.plugin.remove_component(core, button, name)
     }
 
+    fn paste_component(&self, core: CoreHandle, reference_button: &Button, new_button: &mut Button) {
+        self.plugin.paste_component(core, reference_button, new_button)
+    }
+
     fn component_values(&self, core: CoreHandle, button: &Button, name: &str) -> Vec<UIValue> {
         self.plugin.component_values(core, button, name)
     }
@@ -57,17 +67,44 @@ impl SDModule for PluginProxy {
         self.plugin.listening_for()
     }
 
-    fn event(&self, core: CoreHandle, event: SDEvent) {
-        if core.check_for_feature("events") {
+    fn settings(&self, core: Arc<CoreManager>) -> Vec<UIValue> {
+        self.plugin.settings(core)
+    }
+
+    fn set_setting(&self, core: Arc<CoreManager>, value: Vec<UIValue>) {
+        self.plugin.set_setting(core, value)
+    }
+
+    fn global_event(&self, event: SDGlobalEvent) {
+        if check_feature_list_for_feature(&self.metadata.used_features, "global_events") {
+            self.plugin.global_event(event)
+        }
+    }
+
+    fn event(&self, core: CoreHandle, event: SDCoreEvent) {
+        if core.check_for_feature("core_events") {
             self.plugin.event(core, event)
         }
     }
 
+    fn render(&self, core: CoreHandle, button: &UniqueButton, frame: &mut DynamicImage) {
+        if core.check_for_feature("rendering") {
+            self.plugin.render(core, button, frame)
+        }
+    }
+
+    fn render_hash(&self, core: CoreHandle, button: &UniqueButton, hash: &mut Box<dyn Hasher>) {
+        if core.check_for_feature("rendering") {
+            self.plugin.render_hash(core, button, hash)
+        }
+    }
+
     fn metadata(&self) -> PluginMetadata {
         self.metadata.clone()
     }
 }
 
+/// Returns error if plugin is incompatible
 pub fn compare_plugin_versions(versions: &Vec<(String, String)>) -> Result<(), PluginError> {
     let core_versions = SUPPORTED_FEATURES.clone().into_iter()
         .map(|(n, v)| (*n, *v))
@@ -86,6 +123,7 @@ pub fn compare_plugin_versions(versions: &Vec<(String, String)>) -> Result<(), P
     Ok(())
 }
 
+/// Warns about essential features
 pub fn warn_about_essential_features(module: UniqueSDModule) {
     let name = &module.name();
     let features = module.metadata().used_features;
@@ -94,7 +132,8 @@ pub fn warn_about_essential_features(module: UniqueSDModule) {
     warn_for_feature(name, &features, "sdmodule_trait");
 }
 
-pub fn load_plugin<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_manager: Arc<SocketManager>, path: T) -> Result<(), PluginError> {
+/// Loads a plugin into module manager
+pub fn load_plugin<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_manager: Arc<SocketManager>, render_manager: Arc<RenderingManager>, path: T) -> Result<(), PluginError> {
     // Loading file as a library, error if cannot load
     let wrapper: Container<PluginApi> = unsafe { Container::load(path) }?;
 
@@ -102,11 +141,8 @@ pub fn load_plugin<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_m
     let metadata = wrapper.get_metadata();
     compare_plugin_versions(&metadata.used_features)?;
 
-    // Running register function, so plugin can hook their listeners to various parts of the daemon
-    wrapper.register(socket_manager);
-
     // Attempting to get module from the plugin
-    let module: BoxedSDModule = unsafe { Box::from_raw(wrapper.get_module()) };
+    let module: BoxedSDModule = unsafe { Box::from_raw(wrapper.get_module(socket_manager, render_manager)) };
 
     // Wrapping plugin's module into a wrapper that contains loaded library
     let module_proxy: UniqueSDModule = Arc::new(Box::new(PluginProxy { wrapper, metadata, plugin: module }));
@@ -129,7 +165,8 @@ pub fn load_plugin<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_m
     }
 }
 
-pub fn load_plugins_from_folder<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_manager: Arc<SocketManager>, path: T) {
+/// Loads plugins into module manager from path
+pub fn load_plugins_from_folder<T: AsRef<OsStr>>(module_manager: Arc<ModuleManager>, socket_manager: Arc<SocketManager>, render_manager: Arc<RenderingManager>, path: T) {
     let path = Path::new(&path);
     match fs::read_dir(path) {
         Ok(read_dir) => {
@@ -139,7 +176,7 @@ pub fn load_plugins_from_folder<T: AsRef<OsStr>>(module_manager: Arc<ModuleManag
                         if entry.path().is_file() {
                             if let Some(file_name) = entry.path().file_name() {
                                 log::info!("Loading plugin {:?}", file_name);
-                                match load_plugin(module_manager.clone(), socket_manager.clone(), entry.path()) {
+                                match load_plugin(module_manager.clone(), socket_manager.clone(), render_manager.clone(), entry.path()) {
                                     Err(err) => match err {
                                         PluginError::LoadError(err) => log::error!("Failed to load plugin: {}", err),
                                         PluginError::WrongVersion(plugin, software) => log::error!("Failed to load plugin: Plugin is using unsupported version of '{}', software's using '{}'", plugin, software),
@@ -166,6 +203,7 @@ pub fn load_plugins_from_folder<T: AsRef<OsStr>>(module_manager: Arc<ModuleManag
     }
 }
 
+/// Enum for anything wrong that might happen during plugin loading
 #[derive(Debug)]
 pub enum PluginError {
     LoadError(dlopen::Error),
