@@ -2,10 +2,12 @@
 
 use std::io::Write;
 use std::ops::Deref;
-use std::sync::{Arc, RwLock};
+use std::sync::{Arc, Condvar, Mutex, RwLock};
+use std::thread::spawn;
 use serde::{Deserialize, Serialize};
 use serde::de::{DeserializeOwned, Error};
 use serde_json::Value;
+use crate::modules::events::SDGlobalEvent;
 
 /// Type for listener's socket handles
 pub type SocketHandle<'a> = &'a mut dyn Write;
@@ -26,6 +28,7 @@ pub struct SocketPacket {
 
 /// Socket listener, something that can listen in to socket connections
 pub trait SocketListener {
+    /// Called when message is received, handle can be used to send back a response
     fn message(&self, socket: SocketHandle, packet: SocketPacket);
 }
 
@@ -69,7 +72,7 @@ pub fn send_packet<T: SocketData + Serialize>(handle: SocketHandle, previous_pac
         data: Some(serde_json::to_value(data)?)
     };
 
-    write_in_chunks(handle, format!("{}\u{0004}", serde_json::to_string(&packet)?))?;
+    send_packet_as_is(handle, packet)?;
 
     Ok(())
 }
@@ -82,7 +85,7 @@ pub fn send_packet_with_requester<T: SocketData + Serialize>(handle: SocketHandl
         data: Some(serde_json::to_value(data)?)
     };
 
-    write_in_chunks(handle, format!("{}\u{0004}", serde_json::to_string(&packet)?))?;
+    send_packet_as_is(handle, packet)?;
 
     Ok(())
 }
@@ -95,7 +98,14 @@ pub fn send_no_data_packet_with_requester<T: SocketData>(handle: SocketHandle, r
         data: None
     };
 
-    write_in_chunks(handle, format!("{}\u{0004}", serde_json::to_string(&packet)?))?;
+    send_packet_as_is(handle, packet)?;
+
+    Ok(())
+}
+
+/// Sends a packet as is
+pub fn send_packet_as_is(handle: SocketHandle, data: SocketPacket) -> Result<(), SocketError> {
+    write_in_chunks(handle, format!("{}\u{0004}", serde_json::to_string(&data)?))?;
 
     Ok(())
 }
@@ -122,13 +132,15 @@ impl From<std::io::Error> for SocketError {
 /// Manager of socket listeners
 pub struct SocketManager {
     listeners: RwLock<Vec<BoxedSocketListener>>,
+    pools: RwLock<Vec<Arc<SocketPool>>>
 }
 
 impl SocketManager {
     /// Creates a new socket manager
     pub fn new() -> Arc<SocketManager> {
         Arc::new(SocketManager {
-            listeners: RwLock::new(vec![])
+            listeners: Default::default(),
+            pools: Default::default()
         })
     }
 
@@ -137,10 +149,88 @@ impl SocketManager {
         self.listeners.write().unwrap().push(listener);
     }
 
-    /// Send a message event to all listeners
-    pub fn message(&self, handle: SocketHandle, packet: SocketPacket) {
+    /// Sends a message to all listeners, for socket implementation to trigger all listeners when message is received
+    pub fn received_message(&self, handle: SocketHandle, packet: SocketPacket) {
         for listener in self.listeners.read().unwrap().deref() {
             listener.message(handle, packet.clone());
         }
     }
+
+    /// Creates a new message pool
+    pub fn get_pool(&self) -> Arc<SocketPool> {
+        let mut pools = self.pools.write().unwrap();
+
+        let new_pool = Arc::new(SocketPool {
+            messages: Mutex::new(vec![]),
+            condvar: Default::default(),
+            is_open: RwLock::new(true)
+        });
+
+        pools.push(new_pool.clone());
+
+        new_pool
+    }
+
+    /// For listeners or modules to send messages to all active socket connections, for event purposes
+    pub fn send_message(&self, packet: SocketPacket) {
+        let mut pools = self.pools.write().unwrap();
+
+        pools.retain(|x| x.is_open());
+
+        for pool in pools.iter() {
+            pool.add_message(packet.clone())
+        }
+    }
+}
+
+/// Sends packet to all socket connections in different thread, so current thread won't have to wait for write locks
+pub fn send_socket_message(socket_manager: &Arc<SocketManager>, packet: SocketPacket) {
+    let socket_manager = socket_manager.clone();
+    spawn(move || {
+        socket_manager.send_message(packet);
+    });
+}
+
+pub fn send_event_to_socket(socket_manager: &Arc<SocketManager>, event: SDGlobalEvent) {
+    send_socket_message(socket_manager, SocketPacket {
+        ty: "event".to_string(),
+        requester: None,
+        data: Some(serde_json::to_value(event).unwrap())
+    })
+}
+
+/// Pool of messages for socket implementations
+pub struct SocketPool {
+    messages: Mutex<Vec<SocketPacket>>,
+    condvar: Condvar,
+    is_open: RwLock<bool>
+}
+
+impl SocketPool {
+    /// Puts message into the pool
+    pub fn add_message(&self, message: SocketPacket) {
+        let mut messages = self.messages.lock().unwrap();
+        messages.insert(0, message);
+        self.condvar.notify_all();
+    }
+
+    /// Retrieves a message, will block if pool is currently empty
+    pub fn take_message(&self) -> SocketPacket {
+        let mut guard = self.condvar.wait_while(self.messages.lock().unwrap(), |x| x.len() <= 0).unwrap();
+
+        if let Some(packet) = guard.pop() {
+            packet
+        } else {
+            drop(guard);
+            self.take_message()
+        }
+    }
+
+    pub fn is_open(&self) -> bool {
+        *self.is_open.read().unwrap()
+    }
+
+    pub fn close(&self) {
+        *self.is_open.write().unwrap() = false;
+    }
 }
\ No newline at end of file
